<?php

namespace App\Http\Controllers\Api\Admin;

use App\Http\Controllers\Controller;
use App\Http\Requests\EventRequest;
use App\Http\Resources\EventListResource;
use App\Http\Resources\EventResource;
use App\Models\Event;
use Illuminate\Http\Request;
use Illuminate\Http\UploadedFile;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\URL;
use Illuminate\Support\Str;
use Date;

class EventController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $perPage = request('per_page', 10);
        $sortField = request('sort_field', 'created_at');
        $sortDirection = request('sort_direction', 'desc');

        $query = Event::query()
            ->where('status',1)
            ->orderBy($sortField, $sortDirection)
            ->paginate($perPage);

        return EventListResource::collection($query);
    }


    public function listEvents(){
      return Event::where('status',1)->get();
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function store(EventRequest $request)
    {
        $data = $request->validated();
        $data['status'] =1;
        /** @var \Illuminate\Http\UploadedFile $image */
        $logo = $data['logo'] ?? null;
        if ($logo) {
            $logoName = time() . '.' . $logo->getClientOriginalExtension();
            $relPathAndName = $logo->storePubliclyAs('/public/logo', $logoName);
            $data['logo'] = str_replace('public', 'storage', $relPathAndName);
        }

         /** @var \Illuminate\Http\UploadedFile $image */
         $banner = $data['banner'] ?? null;
         if ($banner) {
            $bannerName = time() . '.' . $banner->getClientOriginalExtension();
            $relPathAndName = $banner->storePubliclyAs('/public/banner', $bannerName);
            $data['banner'] = str_replace('public', 'storage', $relPathAndName);
         }

        $event = Event::create($data);

        return response()->json(['status'=>true,'event'=> new EventResource($event)]);
    }

    /**
     * Display the specified resource.
     *
     * @param \App\Models\Event $Event
     * @return \Illuminate\Http\Response
     */
    public function show(Event $event)
    {
        return new EventResource($event);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @param \App\Models\Event      $event
     * @return \Illuminate\Http\Response
     */
    public function update(EventRequest $request, Event $event)
    {
        $data = $request->validated();
        $data['updated_at'] = new Date();

        /** @var \Illuminate\Http\UploadedFile $image */
        $logo = $data['logo'] ?? null;
        if ($logo instanceof UploadedFile &&  $logo->getMimeType() && strpos($logo->getMimeType(), 'image') === 0) {
            $logoName = time() . '.' . $logo->getClientOriginalExtension();
            $relPathAndName = $logo->storePubliclyAs('/public/logo', $logoName);
            $data['logo'] = str_replace('public', 'storage', $relPathAndName);
            if ($event->logo) {
                Storage::deleteDirectory('/public/' . dirname($event->logo));
            }
        }

        /** @var \Illuminate\Http\UploadedFile $image */
        $banner = $data['banner'] ?? null;
        if ($banner instanceof UploadedFile &&  $banner->getMimeType() && strpos($banner->getMimeType(), 'image') === 0) {
          $bannerName = time() . '.' . $banner->getClientOriginalExtension();
          $relPathAndName = $banner->storePubliclyAs('/public/banner', $bannerName);
          $data['banner'] = str_replace('public', 'storage', $relPathAndName);
          if ($event->banner) {
              Storage::deleteDirectory('/public/' . dirname($event->banner));
          }
        }

        $event->update($data);
        return new EventResource($event);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param \App\Models\Event $event
     * @return \Illuminate\Http\Response
     */
    public function destroy(Event $event)
    {
        $event->status = 0;
        if($event->save()){
          return response(true);
        }else{
          return response(false);
        }
    }

    private function saveImage(UploadedFile $image)
    {
        $path = 'images/' . Str::random();
        if (!Storage::exists($path)) {
            Storage::makeDirectory($path, 0755, true);
        }
        if (!Storage::putFileAS('public/' . $path, $image, $image->getClientOriginalName())) {
            throw new \Exception("Unable to save file \"{$image->getClientOriginalName()}\"");
        }

        return $path . '/' . $image->getClientOriginalName();
    }
}
